﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;


namespace PhidgetsControlLibrary {
	public partial class CustomScroll : UserControl {

		double minValue = -1;
		double maxValue = -1;
		double realValue = -1;
		double logBase = 0.00000005; //controls the slope of the log funciton.  very aggressive here so that the bulk of the scale is usable and only the last half jumps quickly.
		double[] dataIntervalValues;

		public CustomScroll() {
			InitializeComponent();
			Unit = "";
		}

		public event EventHandler LogScroll {
			add { slider.Scroll += value; }
			remove { slider.Scroll -= value; }
		}

		public event MouseEventHandler LogMouseUp {
			add { slider.MouseUp += value; }
			remove { slider.MouseUp -= value; }
		}

		public event EventHandler SetClick
        {
            add { setBtn.Click += value; }
            remove { setBtn.Click -= value; }
        }

		public event KeyEventHandler SetEnter
        {
			add { trkTxt.KeyDown += value; }
			remove { trkTxt.KeyDown -= value; }
		}

		#region Properties

		public double Minimum {
			get {
				return this.minValue;
			}
			set {
				slider.Minimum = 0;
				this.minValue = value;
			}
		}

		public double Maximum {
			get {
				return this.maxValue;
			}
			set { //slider should have 100 positions that range between minValue and maxValue
				slider.Maximum = 99;
				this.maxValue = value;
			}
		}

		public double LogBase
        {
            get
            {
				return this.logBase;
            }
            set
            {
				this.logBase = value;
            }
        }

		public bool isIntValue { get; set; }

		public bool isFixedRange { get; set; }

		public string Unit { get; set; }

		public string labelTxt {
			get {
				return trkName.Text;
			}
			set {
				trkName.Text = value;
			}
		}

		public string minTxt {
			get {
				return minLbl.Text;
			}
			set {
				minLbl.Text = value;
			}
		}

		public string maxTxt {
			get {
				return maxLbl.Text;
			}
			set {
				maxLbl.Text = value;
			}
		}

		public string trkText {
			get {
				return trkTxt.Text;
			}
			set {
				trkTxt.Text = value;
			}
		}

		public double Value {
			get {
				return this.realValue;
			}
			set {
				if (value <= maxValue && value >= minValue)
				{
					realValue = value;
					updateSliderFromValue();
					updateTxt();
				}
			}
		}

		public void initializeRange(double min, double max)
        {
			if(isFixedRange == true)
            {
				dataIntervalValues = new double[] {min, min+1, min+2, min+4, min+8, min+16, min+24, min+32, min+40, min+50,
												   min+100, min+200, min+300, min+400, min+500, min+600, min+700, min+800, min+900, min+1000,
												   min+1100, min+1200, min+1300, min+1400, min+1500, min+1600, min+1700, min+1800, min+1900, min+2000,
												   min+2100, min+2200, min+2300, min+2400, min+2500, min+2600, min+2700, min+2800, min+2900, min+3000,
												   min+3100, min+3200, min+3300, min+3400, min+3500, min+3600, min+3700, min+3800, min+3900, min+4000,
												   min+4100, min+4200, min+4300, min+4400, min+4500, min+4600, min+4700, min+4800, min+4900, min+5000,
												   min+5100, min+5200, min+5300, min+5400, min+5500, min+5600, min+5700, min+5800, min+5900, min+6000,
												   min+10000, min+12000, min+14000, min+16000, min+18000, min+20000, min+22000, min+24000, min+26000, min+28000,
												   min+30000, min+32000, min+34000, min+3600, min+38000, min+40000, min+42000, min+44000, min+46000, min+48000,
												   min+50000, min+52000, min+54000, min+56000, min+58000, min+58500, min+59000, min+59500, max-1, max};
				minLbl.Text = min.ToString();
				maxLbl.Text = max.ToString();
            }
            else { throw new Exception(@"[Range not user settable in this mode]"); }
        }

		private void updateTxt() {

			if (!isIntValue) {
				if (maxValue <= 5)
					trkTxt.Text = realValue.ToString("F3") + Unit;
				else
					trkTxt.Text = realValue.ToString("F2") + Unit;
			} else {
				if (maxValue <= 5)
					trkTxt.Text = realValue.ToString("F0") + Unit;
				else
					trkTxt.Text = realValue.ToString("F0") + Unit;
			}
		}

		bool _updatingSlider = false;
		private void updateValueFromSlider() {

			if (isFixedRange == false)
			{
				if (minValue != maxValue)
					realValue = Math.Log(((slider.Value) * (logBase - 1)) / slider.Maximum + 1, logBase) * (maxValue - minValue) + minValue;
			}
			else if (isFixedRange == true)
            {
				if (minValue != maxValue)
					realValue = dataIntervalValues[slider.Value];
            }
		}
		private void updateSliderFromValue() {

			_updatingSlider = true;
			if (isFixedRange == false)
			{
				if (minValue != maxValue)
					slider.Value = (int)((slider.Maximum * (Math.Pow(logBase, -(realValue - minValue) / (minValue - maxValue)) - 1)) / (logBase - 1));
			}
			else if(isFixedRange == true)
			{
				if (minValue != maxValue)
				{
					double closest = dataIntervalValues[0];
					int index = -1;
					for(int i=0;i<dataIntervalValues.Length;i++)
                    {
						if (Math.Abs(dataIntervalValues[i] - realValue) <= Math.Abs(closest - realValue))
						{
							closest = dataIntervalValues[i];
							index = i;
						}
                    }
					slider.Value = index;
				}
			}
			_updatingSlider = false;
		}

		#endregion

		private void slider_Scroll(object sender, EventArgs e) {

			if (_updatingSlider)
				return;

			updateValueFromSlider();
			updateTxt();
		}

        private void setBtn_Click(object sender, EventArgs e)
        {
			try
			{
				this.Value = double.Parse(trkTxt.Text);
			}
			catch(Exception ex)
			{ this.trkTxt.Text = "N/A"; }
        }

        private void trkTxt_KeyDown(object sender, KeyEventArgs e)
        {
			if (e.KeyCode == Keys.Enter)
			{
				try
				{
					this.Value = double.Parse(trkTxt.Text);
					e.Handled = true;
					e.SuppressKeyPress = true;
				}
				catch(Exception ex)
                {
					this.trkTxt.Text = "N/A";
                }
			}
        }

        private void trkTxt_MouseClick(object sender, MouseEventArgs e)
        {
			trkTxt.SelectAll();
        }
    }
}
